package uk.ac.open.kmi.owl;

import java.net.URI;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.semanticweb.owl.model.OWLAnnotationProperty;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDataFactory;
import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.change.AddAnnotationInstance;
import org.semanticweb.owl.model.change.AddEntity;
import org.semanticweb.owl.model.change.ChangeVisitor;
import org.semanticweb.owl.model.change.OntologyChange;
import org.semanticweb.owl.util.OWLConnection;
import org.semanticweb.owl.util.OWLManager;

import uk.ac.open.kmi.skos.SKOSConcept;

/**
 * OWLOntologyBuilder is used to create and maintain the ontology for the alignment
 */
public class OWLOntologyBuilder {
    private OWLOntology onto=null;
    private OWLConnection conn = null;
    private OWLDataFactory fact=null;
    private ChangeVisitor visitor=null;
    
    private String uriStr=null;
    
    public OWLOntologyBuilder(String uriStr){
        
        try  {
            this.uriStr=uriStr;
            Map parameters = new HashMap();
            conn = OWLManager.getOWLConnection();
            fact = conn.getDataFactory();

            URI uri = new URI(uriStr);
            onto= conn.createOntology( uri, uri );
            
            visitor = conn.getChangeVisitor( onto );
            OWLClass owlClass =  fact.getOWLClass( new URI(uriStr+"#Thing") );
            OntologyChange oc = new AddEntity( onto,owlClass, null );
            oc.accept( visitor );


        } catch (Exception ex)  {
            ex.printStackTrace();
        }   
    }
    
    /**
     * Creates an OWL class from the SKOS concept
     * The following mappings have been assumed
     * skos:Concept -> owl:Class
     * skos:prefLabel -> rdfs:label
     * skos:altLabel -> rdfs:label
     * skos:broader -> rdfs:subClassOf
     * skos:narrower -> No need to handle in the class since it will be an 
     * another class like owl:Class with rdfs:subClassOf this class
     * skos:definition -> rdfs:comment
     * skos:scopeNote -> rdfs:comment
     *
     * @param skosConcept
     */
    public void createClass(SKOSConcept skosConcept){
        OWLClass owlClass=null;
        OWLAnnotationProperty prop=null;
        AddAnnotationInstance aai=null;
        String str=null;
        try  {
        if (skosConcept.getAbout().indexOf(getUriStr())!=-1)
            owlClass =  fact.getOWLClass( new URI(skosConcept.getAbout()) );
        else
            owlClass =  fact.getOWLClass( new URI(uriStr+"#"+skosConcept.getAbout()) );
              
            for (Iterator iter=skosConcept.getPrefLabels().iterator();iter.hasNext();){
                str=(String)iter.next();
                prop = fact.getOWLAnnotationProperty( new URI("http://www.w3.org/2000/01/rdf-schema#label") ); 
                aai=new AddAnnotationInstance( onto,owlClass, prop, str, null ); 
                aai.accept( visitor ); 
            }
            for (Iterator iter=skosConcept.getAltLabels().iterator();iter.hasNext();){
                str=(String)iter.next();
                prop=fact.getOWLAnnotationProperty( new URI("http://www.w3.org/2000/01/rdf-schema#label") ); 
                aai=new AddAnnotationInstance( onto,owlClass, prop, str, null ); 
                aai.accept( visitor ); 
            }
            for (Iterator iter=skosConcept.getBroader().iterator();iter.hasNext();){
                str=(String)iter.next();
                prop=fact.getOWLAnnotationProperty( new URI("http://www.w3.org/2000/01/rdf-schema#subClassOf") ); 
                aai=new AddAnnotationInstance( onto,owlClass, prop, str, null ); 
                aai.accept( visitor ); 
            }
            for (Iterator iter=skosConcept.getDefinition().iterator();iter.hasNext();){
                str=(String)iter.next();
                prop=fact.getOWLAnnotationProperty( new URI("http://www.w3.org/2000/01/rdf-schema#comment") ); 
                aai=new AddAnnotationInstance( onto,owlClass, prop, str, null ); 
                aai.accept( visitor ); 
            }
            for (Iterator iter=skosConcept.getScopeNote().iterator();iter.hasNext();){
                str=(String)iter.next();
                prop=fact.getOWLAnnotationProperty( new URI("http://www.w3.org/2000/01/rdf-schema#comment") ); 
                aai=new AddAnnotationInstance( onto,owlClass, prop, str, null ); 
                aai.accept( visitor ); 
            }
            OntologyChange oc = new AddEntity( onto,owlClass, null );
            oc.accept( visitor );   
 
            
        } catch (Exception ex)  {
            ex.printStackTrace();
        }   
    }
    
    /**
     * Get a particular class from the Ontology
     * @param className
     * @return OWLClass
     */
    public OWLClass getClass(String className){
        OWLClass ret=null;
        try  {
            URI uri = new URI(uriStr+"#"+className);
            onto.getClass(uri);
        } catch (Exception ex)  {
            ex.printStackTrace();
        } 
        return ret;
        
    }


    public OWLOntology getOntology() {
        return onto;
    }

    public String getUriStr() {
        return uriStr;
    }
}
